local _, L = ...;

-- ---------------------------------------------------------------------------------------
-- Stoen Guard Danger Announce
-- usefule only if applying the packed Strategie on the stone guardians (all three guardians
-- tanked together).
-- It will tell you if the next overload will be harmless (next boss to overload is also
-- petrifying) or not.
-- ---------------------------------------------------------------------------------------
-- ---------------------------------------------------------------------------------------
-- LOCAL VARIABLES
-- ---------------------------------------------------------------------------------------
local showenable=0
local bossPower={0,0,0} --current power of boss1,...,boss4
local numberpetrifying --number of the boss that is petrifying
local combatStarted=0
local testMode=false
local total=0
local change=true
local previousOverloadingBoss
local version = GetAddOnMetadata("Stone Guard Danger Announce","Version") or 0
local wispRaid
local loaded

-- ---------------------------------------------------------------------------------------
-- Handle Default Loading of saved variables.
-- ---------------------------------------------------------------------------------------
if not sdaSavedOpt then
		sdaSavedOpt = {
			lockWindow = true,
			displayRaidWarning = false 
		}
end

-- ---------------------------------------------------------------------------------------
-- Frame setup
-- Used to display messages.
-- ---------------------------------------------------------------------------------------
-- Dummy Frame to catch "PLAYER_ENTERING_WORLD event and display welcome message. --------
local welcomeMessageFrame = CreateFrame("Frame")
welcomeMessageFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
welcomeMessageFrame:SetScript("OnEvent", function()
	if not loaded then
		print("|c001E90FFStone|r|c00008000Guard|r|c00FF4000Danger|r|c00FF44FFAnnounce|r loaded. \"|c00FFFF00/sda help|r\".")

		loaded=true
	end
end)

-- Main Frame ----------------------------------------------------------------------------
local SDAFrame = CreateFrame("Frame","SGDA",UIParent)
local announceText = SDAFrame:CreateFontString("next_target_text")

announceText:SetFont("Fonts\\ARIALN.TTF", 16, "OUTLINE") --TEXT SIZE
announceText:SetPoint("CENTER", SDAFrame, "CENTER",5,-5)

SDAFrame:EnableMouse(false)
SDAFrame:SetMovable(true)
SDAFrame:SetUserPlaced(true)
SDAFrame:SetResizable(true)
SDAFrame:SetScript("OnMouseDown", function()
	SDAFrame:StartMoving()
end)
SDAFrame:SetScript("OnMouseUp", function ()
	if testMode then
		if change then
			SDAFrame.t:SetTexture(0, 255, 0)
			announceText:SetText(L["NO DAMAGE"])
		else
			SDAFrame.t:SetTexture(255, 0, 0)
			announceText:SetText(L["! DAMAGE ! DAMAGE ! DAMAGE !"])
		end

		change = not change
	end

	SDAFrame:StopMovingOrSizing()
end)
SDAFrame:SetSize(515,45) --FRAME SIZE (width, height)
SDAFrame:SetFrameStrata("BACKGROUND")
SDAFrame:SetPoint("CENTER")
SDAFrame.t = SDAFrame:CreateTexture()
SDAFrame.t:SetAllPoints()
SDAFrame.t:SetTexture(0, 0, 0)
SDAFrame.t:SetAlpha(0.5)
SDAFrame:Hide()

-- ---------------------------------------------------------------------------------------
-- FUNCTIONS.
-- ---------------------------------------------------------------------------------------
--[[ DESCRIPTION ---------------------------------------------------------------------------
--       Update variables used to calculate if the next overload is dangerous or not.
--
-- PARAMETERS
--
-- RETURNS
--
-- --------------------------------------------------------------------------------------- ]]
local function update()

	-- Display message Frame
	SDAFrame:Show()

	-- Update Bosses Buffs
	bossPower={UnitPower("boss1"),UnitPower("boss2"),UnitPower("boss3")}

end

--[[ DESCRIPTION ---------------------------------------------------------------------------
--       Function associated with the start of the fight.
--
-- PARAMETERS
--
-- RETURNS
--
-- --------------------------------------------------------------------------------------- ]]
local function combat_start_func()
	if UnitGUID("boss1") and (UnitGUID("boss1"):sub(6,10)=="0EA93" or UnitGUID("boss1"):sub(6,10)=="0EA8B" or UnitGUID("boss1"):sub(6,10)=="0EA0B" or UnitGUID("boss1"):sub(6,10)=="0EA8F") then
		combatStarted = 1
		SDAFrame:RegisterEvent("UNIT_POWER")
		SDAFrame:EnableMouse(false)
		showenable=1
	end
end

--[[ DESCRIPTION ---------------------------------------------------------------------------
--       Function associated with the end of the fight.
--
-- PARAMETERS
--
-- RETURNS
--
-- --------------------------------------------------------------------------------------- ]]
local function combat_end_func()
	if combatStarted==1 then
		SDAFrame:UnregisterEvent("UNIT_POWER")
		SDAFrame:Hide()
		showenable=0
		firstcycle=50
	end
	combatStarted=0
end

--[[ DESCRIPTION ---------------------------------------------------------------------------
--       Tests is the specified boss as the Buff "Real Form" active meaning that it
-- is stakcing.
--
-- PARAMETERS
--      * localBossName : name of the boss to test.
--
-- RETURNS
--      * True is boss is in Real Form
--      * False otherwise.
-- --------------------------------------------------------------------------------------- ]]
local function isRealForm(localBossName)

	local name,rank,iconTexture,count,debuffType,duration,expirationTime,source,isStealable,shouldConsolidate,spellID=UnitBuff(localBossName, 1)
	if spellID=="115827" or spellID=="115828" or spellID=="115829" or spellID=="115771" then
		return true
	end

	return false

end

--[[ DESCRIPTION ---------------------------------------------------------------------------
--      Tests it the specified Boss is currently petrifying or not.
-- PARAMETERS
--      * Boss name to test.
-- RETURNS
--      * True if boss is petryfying
--      * False otherwise.
-- --------------------------------------------------------------------------------------- ]]
local function isPetrifying(localBossName)

	if UnitBuff("boss1",2) then
		numberpetrifying=1
	elseif UnitBuff("boss2",2) then
		numberpetrifying=2
	elseif UnitBuff("boss3",2) then
		numberpetrifying=3
	else
		numberpetrifying=nil
	end

	if numberpetrifying then
		if "boss"..numberpetrifying==localBossName then
			return true
		else
			return false
		end
	else
		return false
	end
end

--[[ DESCRIPTION ---------------------------------------------------------------------------
--      Tests is the specified Boss is dangerous or not. Meaning is it is the next to
-- overload and does not have the petrifying debuff.
--
-- PARAMETERS
--      * Boss name to test.
-- RETURNS
--      * True if boss is next to overload and NOT petrifying
--      * False otherwise.
-- --------------------------------------------------------------------------------------- ]]
local function nextToOverload()
	if bossPower[1] > bossPower[2] and bossPower[1] > bossPower[3] then
		return "boss1"
	elseif bossPower[2] > bossPower[1] and bossPower[2] > bossPower[3] then
		return "boss2"
	elseif bossPower[3] > bossPower[1] and bossPower[3] > bossPower[2] then
		return "boss3"
	end

	-- ----------------------------------
	-- No Boss to overload.
	-- ----------------------------------
	return nil
end

--[[ DESCRIPTION ---------------------------------------------------------------------------
--      Tests if the specified Boss is dangerous or not. Meaning is it is the next to
-- overload and does not have the petrifying debuff.
--
-- PARAMETERS
--      * Boss name to test.
-- RETURNS
--      * True if boss is next to overload and NOT petrifying
--      * False otherwise.
-- --------------------------------------------------------------------------------------- ]]
local function isDangerousBoss(localBossName)
	if isRealForm(localBossName) and not isPetrifying(localBossName) then
		return true
	else
		return false
	end
end

--[[ DESCRIPTION ---------------------------------------------------------------------------
--		Print messages in the frame and on the raid warning channel if the configuration
-- allows it.
--
-- PARAMETERS
-- 		stringToPrint : Character string to print in the message window and on the Raid
-- Warning channel if configuration allows it.
--
-- RETURNS
--
-- --------------------------------------------------------------------------------------- ]]
local function printMessage(stringToPrint)
	if wispRaid==1 then
		SendChatMessage (stringToPrint, "RAID_WARNING");
		SendChatMessage (stringToPrint, "RAID_WARNING");
	end
	announceText.SetText(stringToPrint);
end

--[[ DESCRIPTION ---------------------------------------------------------------------------
--       Update variables used to calculate if the next overload is dangerous or not.
--
-- PARAMETERS
--
-- RETURNS
--
-- --------------------------------------------------------------------------------------- ]]
local function check()
	if combatStarted then
		-- ------------------------------------------
		-- Update current Boss Buffs.
		-- ------------------------------------------
		update()

		-- ------------------------------------------
		-- Identify which boss is going to overload.
		-- ------------------------------------------
		local nextBossToOverload = nextToOverload()

		-- -------------------------------------------
		-- Save the name of the overloading boss and
		-- only prints if the boss changed compared to 
		-- previous test. If Waiting then the status
		-- Will have changed from a value to nil.
		-- -------------------------------------------
		previousOverloadingBoss = nextBossToOverload

		-- -------------------------------------------
		-- Action only if Overloading Boss changed.
		-- -------------------------------------------
		if not nextBossToOverload == previousOverloadingBoss then
			if nextBossToOverload then
				if numberpetrifying then
					-- ------------------------------------------------------
					-- There is a boss petryfying, determine if it
					-- is the next to overload (harmless) or not (dangerous)
					-- or not, and print a different message depending on the
					-- outcome.
					-- ------------------------------------------------------
					if isDangerousBoss(nextBossToOverload) then
						printMessage("{rt8} "..L["! DAMAGE ! DAMAGE ! DAMAGE !"].." {rt8}")
					else
						printMessage("{rt2} "..L["NO DAMAGE"].." {rt2}")
					end
				else
					-- ------------------------------------------------------
					-- No BOSS is petrifying, waiting for next one.
					-- ------------------------------------------------------
					printMessage("{rt1} "..L["WAITING FOR PETRIFICATION"].." {rt1}")
				end
			end
		end
	end
end

-- ---------------------------------------------------------------------------------------
-- REGISTER EVENTS
-- ---------------------------------------------------------------------------------------
local combat_start = CreateFrame("Frame",nil,UIParent)
combat_start:RegisterEvent("PLAYER_REGEN_DISABLED")
combat_start:SetScript("OnEvent", combat_start_func)

local combat_end = CreateFrame("Frame",nil,UIParent)
combat_end:RegisterEvent("PLAYER_REGEN_ENABLED")
combat_end:SetScript("OnEvent", combat_end_func)

-- ---------------------------------------------------------------------------------------
-- Trigger the check everytime an event is thrown.
-- ---------------------------------------------------------------------------------------
SDAFrame:SetScript("OnEvent", check)

-- ---------------------------------------------------------------------------------------
-- Slash Commands
-- ---------------------------------------------------------------------------------------
SLASH_SDA1 = "/sda"
function slashHandler(msg, editbox)
	-- ------------------------------------------
	-- Show Addon window
	-- ------------------------------------------
	-- if msg=='' then
	-- 	if showenable==0 then
	-- 		if SDAFrame:IsShown() then
	-- 			SDAFrame:Hide()
	-- 			SDAFrame:EnableMouse(false)
	-- 		else
	-- 			SDAFrame:Show()
	-- 			SDAFrame:EnableMouse(true)
	-- 		end
	-- 	elseif SDAFrame:IsMouseEnabled() then
	-- 		SDAFrame:EnableMouse(false)
	-- 	else
	-- 		SDAFrame:EnableMouse(true)
	-- 	end
	-- ------------------------------------------
	-- Enter test mode and change message each time
	-- a click is made in the window and we are not
	-- in combat.
	-- ------------------------------------------
	if msg=='test' and combatStarted~=1 then
		testMode = not testMode
		if testMode then
			SDAFrame:Show()
			SDAFrame:EnableMouse(true)
			SDAFrame.t:SetTexture(255, 0, 0)
			announceText:SetText(L["! DAMAGE ! DAMAGE ! DAMAGE !"])
		else
			SDAFrame:Hide()
			SDAFrame:EnableMouse(false)
		end
	-- ------------------------------------------
	-- Open Configuration Window.
	-- ------------------------------------------
	elseif msg=="config" then
		InterfaceOptionsFrame_OpenToCategory("Stone Guard Danger Announce")
		return true
	-- ------------------------------------------
	-- Toggle Raid Warning Messages.
	-- ------------------------------------------
	elseif msg=="rw" then
		sdaSavedOpt.displayRaidWarning = not sdaSavedOpt.displayRaidWarning
		return true
	-- ------------------------------------------
	-- Lock Message Window position.
	-- ------------------------------------------
	elseif msg=="lock" then
		sdaSavedOpt.lockWindow = true
		return true
	-- ------------------------------------------
	-- Unlock Message Window position.
	-- ------------------------------------------
	elseif msg=="unlock" then
		sdaSavedOpt.lockWindow = false
		return true
	-- ------------------------------------------
	-- Display Help Message.
	-- ------------------------------------------
	elseif msg=="help" then
		print("|c001E90FFStone|r|c00008000Guard|r|c00FF4000Danger|r|c00FF44FFAnnounce|r Help.")
		print("--> /sda test : starts test mode.")
		print("--> /sda config : opens config window.")
		print("--> /sda rw : toggle on/off the raid warnings.")
		print("--> /sda lock : lock the message window in place.")
		print("--> /sda unlock : unlock the message window.")
		print("--> /sda help : displays this message.")
	end
end

SlashCmdList["SDA"]=slashHandler
